package be.iminds.ilabt.jfed.restapi.dao;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.FedmonApiCommon;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Location;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Service;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ServiceBuilder;
import org.skife.jdbi.v2.Query;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * DaoCommon: common functions used by all *Dao classes
 */
public class DaoCommon {
    static String makeWherePartList(String varName, int matchValueCount) {
        String res = "";
        boolean first = true;
        for (int i = 0; i < matchValueCount; i++) {
            if (!first)
                res += ",";
            res += "#"+varName+i;
            first = false;
        }
        res += "";
        return res;
    }

    static String makeWherePart(String colName, String varName, int matchValueCount) {
        String res = colName+" IN (";
        res += makeWherePartList(varName, matchValueCount);
        res += ")";
        return res;
    }

    /**
     * joins using AND
     * also adds WHERE itself
     */
    static String joinWhereParts(List<String> whereParts) {
        String res = "";
        if (!whereParts.isEmpty()) {
            res += " WHERE ";
            boolean first = true;
            for (String wherePart : whereParts) {
                if (!first)
                    res += " AND ";
                res += "("+wherePart+")";
                first = false;
            }
            res += " ";
        }
        return res;
    }

    static Query<Map<String, Object>> setVars(Query<Map<String, Object>> q, String varName, List values) {
        int i = 0;
        for (Object val : values) {
            q = q.bind(varName+(i++), val);
        }
        return q;
    }

    static Location toLocation(String columnValue) {
        if (columnValue == null)
            return null;
        return Location.decodeFromString(columnValue);
    }

    static String rowString(Map<String, Object> row, String key) {
        if (!row.containsKey(key))
            throw new RuntimeException("row does not have column \""+key+"\". All columns: "+row.keySet());
        Object res = row.get(key);
        if (res == null)
            return null;
        if (res instanceof String)
            return (String) res;
        throw new RuntimeException("row \""+key+"\" is not a String. It is a "+res.getClass().getName());
    }

    static List<String> rowStringArray(Map<String, Object> row, String key) {
        if (!row.containsKey(key))
            throw new RuntimeException("row does not have column \""+key+"\". All columns: "+row.keySet());
        Object res = row.get(key);
        if (res == null)
            return null;
        if (res instanceof String) {
            String s = (String) res;
            if (s.trim().isEmpty()) return Collections.emptyList();
            String[] rr = s.trim().split(",");
            if (rr.length == 1 && rr[0].trim().isEmpty()) return Collections.emptyList();
            return Arrays.asList(rr);
        }
        throw new RuntimeException("row \""+key+"\" is not a String. It is a "+res.getClass().getName());
    }

    static List<Integer> rowIntegerArray(Map<String, Object> row, String key) {
        if (!row.containsKey(key))
            throw new RuntimeException("row does not have column \""+key+"\". All columns: "+row.keySet());
        Object res = row.get(key);
        if (res == null)
            return null;
        if (res instanceof String) {
            String s = (String) res;
            if (s.trim().isEmpty()) return Collections.emptyList();
            String[] rr = s.trim().split(",");
            if (rr.length == 1 && rr[0].trim().isEmpty()) return Collections.emptyList();
            List<Integer> ia = new ArrayList<>();
            for (String r : rr) {
                int i = Integer.parseInt(r.trim());
                ia.add(i);
            }
            return ia;
        }
        throw new RuntimeException("row \""+key+"\" is not a String. It is a "+res.getClass().getName());
    }

    static Boolean rowBoolean(Map<String, Object> row, String key) {
        if (!row.containsKey(key))
            throw new RuntimeException("row does not have column \""+key+"\". All columns: "+row.keySet());
        Object res = row.get(key);
        if (res == null)
            return null;
        if (res instanceof Boolean)
            return (Boolean) res;
        throw new RuntimeException("row \""+key+"\" is not a Boolean. It is a "+res.getClass().getName());
    }
    static Boolean rowBoolean(ResultSet row, String key) throws SQLException {
        if (row.getObject(key) == null)
            return null;
        return row.getBoolean(key);
    }

    static Integer rowInteger(Map<String, Object> row, String key) {
        if (!row.containsKey(key))
            throw new RuntimeException("row does not have column \""+key+"\". All columns: "+row.keySet());
        Object res = row.get(key);
        if (res == null)
            return null;
        if (res instanceof Integer)
            return (Integer) res;
        if (res instanceof Long)
            return ((Long) res).intValue();
        if (res instanceof Number)
            return ((Number) res).intValue();
        throw new RuntimeException("row \""+key+"\" is not a Integer. It is a "+res.getClass().getName());
    }
    static Integer rowInteger(ResultSet row, String key) throws SQLException {
        if (row.getObject(key) == null)
            return null;
        return row.getInt(key);
    }
    static Long rowLong(ResultSet row, String key) throws SQLException {
        if (row.getObject(key) == null)
            return null;
        return row.getLong(key);
    }

    static List<Service> idsToService(String columnValue) {
        if (columnValue == null) return null;
        return idToService(csvToIntegerArray(columnValue));
    }

    static Service singleIidToService(String columnValue) {
        if (columnValue == null) return null;
        Integer id = Integer.parseInt(columnValue);
        return idToService(id);
    }

    private static List<Service> idToService(List<Integer> serviceIds) {
        if (serviceIds == null) return null;
        return serviceIds.stream().map(i -> idToService(i)).collect(Collectors.toList());
    }

    private static Service idToService(Integer serviceId) {
        if (serviceId == null) return null;
        return new ServiceBuilder().setId(serviceId).createMinimized(FedmonApiCommon.Minimization.ID_ONLY);
    }

    private static List<Integer> csvToIntegerArray(String columnValue) {
        if (columnValue == null)
            return null;
        String s = (String) columnValue;
        if (s.trim().isEmpty()) return Collections.emptyList();
        String[] rr = s.trim().split(",");
        if (rr.length == 1 && rr[0].trim().isEmpty()) return Collections.emptyList();
        List<Integer> ia = new ArrayList<>();
        for (String r : rr) {
            int i = Integer.parseInt(r.trim());
            ia.add(i);
        }
        return ia;
    }

    static List<String> csvToStringArray(String columnValue) {
        if (columnValue == null)
            return null;
        String s = (String) columnValue;
        if (s.trim().isEmpty()) return Collections.emptyList();
        String[] rr = s.trim().split(",");
        if (rr.length == 1 && rr[0].trim().isEmpty()) return Collections.emptyList();
        return Arrays.asList(rr);
    }
}
